User's Guide
------------

.. toctree::
    :maxdepth: 2

    guide

API Reference
-------------

.. toctree::
    :maxdepth: 2

    api_reference

Contributing
------------

If you are interested in contributing to the project please refer to :ref:`contributing`

.. toctree::
    :maxdepth: 2

    contributing

Additional Notes
----------------

Legal information and changelog are here for the interested.

.. toctree::
    :maxdepth: 2

    changelog
    license

Indices and tables
------------------

* :ref:`genindex`
* :ref:`modindex`
* :ref:`search`